/**
 * @file
 * Contains Site Integrator javascript code.
 */
(function ($, Drupal, drupalSettings) {

  'use strict';

  Drupal.site_integrator = {};

  /**
   * Attaches the JS to manage iframe height.
   */
  Drupal.behaviors.siteIntegrator = {

    attach: function (context, settings) {
      once('siteIntegrator', 'html').forEach(function (element) {
        // Load iframe contents.
        // "decoder" is used to decode encoded HTML.
        let decoder = document.createElement('div');
        let client_init_script = drupalSettings.site_integrator.path + '/js/client-init.js';
        for (let frame_id in drupalSettings.site_integrator_sites) {
          let site_frame = document.getElementById(frame_id);
          if (site_frame && !drupalSettings.site_integrator_sites[frame_id]['included']) {
            let raw_content = drupalSettings.site_integrator_sites[frame_id]['site_content'] || 'Not available';
            decoder.innerHTML = raw_content;
            let site_content = decoder.textContent;
            // Append initialization management script to head.
            site_content = site_content.replace(
              /<\/head\s*>/i,
              '<script type="text/javascript" data-frame_id="' + frame_id + '" src="' + client_init_script + '"></script></head>'
            );
            // Manage css, js and regexp filters (selector filters are managed
            // later).
            let filters = drupalSettings.site_integrator_sites[frame_id]['filters'] || [];
            filters.forEach(
              (filter) => {
                let regexp_str = '';
                let replacement = filter.replacement || '';
                if ('css' == filter.type) {
                  regexp_str = '(<link\\s*[^>]*\\shref=)[\'"][^>\'"]*' + filter.filter + '[^>\'"]*[\'"]([^>]*>)';
                  if (replacement) {
                    replacement = '$1"' + replacement + '"$2';
                  }
                }
                else if ('js' == filter.type) {
                  regexp_str = '(<script\\s*[^>]*\\ssrc=)[\'"][^>\'"]*' + filter.filter + '[^>\'"]*[\'"]([^>]*>(?:[^<]*</script>)?)';
                  if (replacement) {
                    replacement = '$1"' + replacement + '"$2';
                  }
                }
                else if ('regexp' == filter.type) {
                  regexp_str = filter.filter;
                }
                if (regexp_str) {
                  const regexp = new RegExp(regexp_str, 'gm');
                  site_content = site_content.replace(regexp, replacement);
                }
              }
            );
            site_frame.contentDocument.write(site_content);
            site_frame.contentDocument.close();
            // @todo See if selector filters could be managed here.
            drupalSettings.site_integrator_sites[frame_id]['included'] = true;
          }
          else {
            console.log(
              'WARNING: site integrator: frame "' + frame_id + '" not found.'
            );
          }
        }
        // Free memory.
        decoder.textContent = '';
        decoder = null;

        // Add listener for integrated sites.
        window.onmessage = function (event) {
          if (event.data
            && ('object' == typeof(event.data))
            && (event.data.frame_id)
          ) {
            let curren_frame_id = event.data.frame_id;
            let site_frame = document.getElementById(curren_frame_id);
            // Keep track of frame window object.
            if (!drupalSettings.site_integrator_sites[curren_frame_id].window) {
              drupalSettings.site_integrator_sites[curren_frame_id].window = event.source;
            }
            if (('resize' == event.data.type) && (event.data.height)) {
              // Manage frame content change resizing event (if not fixed).
              if (!drupalSettings.site_integrator_sites[curren_frame_id].site_height) {
                // We add 20px to get some extra room to avoid a scroll bar.
                site_frame.style.height = (event.data.height + 20) + 'px';
                // Create an observer to dynamically manage changes.
                let observer = new ResizeObserver(function(mutations) {
                  site_frame.style.height = (drupalSettings.site_integrator_sites[curren_frame_id].window.document.getElementsByTagName('body')[0].clientHeight + 20) + 'px';
                });
                observer.observe(drupalSettings.site_integrator_sites[curren_frame_id].window.document.getElementsByTagName('body')[0], {
                    childList: true
                });
              }
            }
            else if ('filter' == event.data.type) {
              //  Manage selector filters.
              let filters = drupalSettings.site_integrator_sites[curren_frame_id].filters || [];
              // "html_parser" is used to parse HTML.
              let html_parser = site_frame.contentDocument.createElement('div');
              filters.forEach(
                (filter) => {
                  if (('selector' == filter.type) && filter.filter) {
                    if (filter.replacement) {
                      site_frame.contentDocument.querySelectorAll(filter.filter).forEach(
                        (item) => {
                          html_parser.innerHTML = filter.replacement;
                          item.replaceWith(...html_parser.children);
                        }
                      );
                    }
                    else {
                      site_frame.contentDocument.querySelectorAll(filter.filter).forEach(
                        (item) => {
                          item.remove();
                        }
                      );
                    }
                  }
                }
              );
            }
          }
        };
      })
    },
    /**
     * Returns frame DOM identifiers in use for a given integrated site.
     *
     * @param (string) base_id
     *   Identifier of the integrated site (ie. Drupal content identifier).
     *
     * @return (Array)
     *   An array of DOM identifiers or an empty array if no corresponding
     *   integration found.
     *   If base_id is not set or false, returns all the integrated site frame
     *   identifiers.
     */
    getFrameId: function (base_id) {
      if (!base_id) {
        return Object.keys(drupalSettings.site_integrator_sites);
      }
      let frame_identifiers = [];
      for (let key in drupalSettings.site_integrator_sites) {
        if (key.startsWith(base_id + '_')) {
          frame_identifiers.push(key);
        }
      }
      return frame_identifiers;
    }
  };

})(jQuery, Drupal, drupalSettings);

