<?php

namespace Drupal\site_integrator\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines an integrated site config.
 *
 * @ingroup site_integrator
 *
 * @ConfigEntityType(
 *   id = "integrated_site",
 *   label = @Translation("Integrated site"),
 *   admin_permission = "administer integrated sites",
 *   handlers = {
 *     "access" = "Drupal\site_integrator\IntegratedSiteAccessController",
 *     "list_builder" = "Drupal\site_integrator\Controller\IntegratedSiteListBuilder",
 *     "form" = {
 *       "add" = "Drupal\site_integrator\Form\IntegratedSiteAddForm",
 *       "edit" = "Drupal\site_integrator\Form\IntegratedSiteEditForm",
 *       "delete" = "Drupal\site_integrator\Form\IntegratedSiteDeleteForm"
 *     }
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label"
 *   },
 *   links = {
 *     "edit-form" = "/integrated-site/manage/{integrated_site}",
 *     "delete-form" = "/integrated-site/manage/{integrated_site}/delete"
 *   },
 *   config_export = {
 *     "id",
 *     "uuid",
 *     "label",
 *     "base_url",
 *     "path",
 *     "height",
 *     "width",
 *     "mode",
 *     "passthrough",
 *     "enabled",
 *     "editable",
 *     "redirections",
 *     "filtering",
 *     "roles"
 *   }
 * )
 */
class IntegratedSite extends ConfigEntityBase implements IntegratedSiteInterface {

  /**
   * The integrated site ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The integrated site UUID.
   *
   * @var string
   */
  protected $uuid;

  /**
   * The integrated site label.
   *
   * @var string
   */
  protected $label;

  /**
   * The integrated site base URL.
   *
   * @var string
   */
  protected $base_url;

  /**
   * The integrated site Drupal path.
   *
   * @var string
   */
  protected $path;

  /**
   * The integrated site integration mode.
   *
   * @var int
   */
  protected $mode;

  /**
   * The integrated site integration iframe default height.
   *
   * @var string
   */
  protected $height;

  /**
   * The integrated site integration iframe default width.
   *
   * @var string
   */
  protected $width;

  /**
   * The list of passthrough URL.
   *
   * @var array
   */
  protected $passthrough;

  /**
   * Enabled status of the integration.
   *
   * @var bool
   */
  protected $enabled;

  /**
   * Redirection pages.
   *
   * Redirections are keyed by the HTTP status code. There are two special
   * redirection keys: 'disabled' is used when the site integration is disabled,
   * and 'forbidden' when the site integration is not allowed to current user.
   * Generic HTTP codes can also be handled by "3xx", "4xx", and "5xx" keys.
   *
   * @var string[]
   */
  protected $redirections;

  /**
   * Filters.
   *
   * Content filtering settings. Each element of the array is an associative
   * array with the following keys:
   * - 'type': string of the type. One of 'css', 'js', or 'regexp' (see
   *   IntegratedSiteInterface::FILTER_TYPE_* constants).
   * - 'filter': a filtering expression.
   * - 'replacement': a replacement expression. Can be empty.
   *
   * @var array
   */
  protected $filtering;

  /**
   * Allowed roles.
   *
   * Roles allowed to use current integrated site.
   *
   * @var string[]
   */
  protected $roles;

  /**
   * {@inheritdoc}
   */
  public function getBaseUrl() :string {
    return $this->base_url ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function setBaseUrl(string $base_url) :self {
    $this->base_url = $base_url;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getPath() :string {
    return $this->path ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function setPath(string $path) :self {
    $this->path = $path;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getIntegrationMode() :int {
    return $this->mode ?? 0;
  }

  /**
   * {@inheritdoc}
   */
  public function setIntegrationMode(int $mode) :self {
    $this->mode = $mode;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getPassthroughList() :array {
    return $this->passthrough ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function setPassthroughList(array $passthrough) :self {
    $this->passthrough = $passthrough;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled() :bool {
    return $this->enabled ?? FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function setEnabled(bool $status = TRUE) :self {
    $this->enabled = $status;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isEditable() :bool {
    return $this->editable ?? TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function setEditable(bool $status = TRUE) :self {
    $this->editable = $status;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirection(?string $code = NULL) :array|string|null {
    if (empty($code)) {
      return $this->redirections;
    }
    return $this->redirections[$code] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setRedirection(string $code, ?string $redirection) :self {
    if (empty($redirection)) {
      unset($this->redirections[$code]);
    }
    else {
      $this->redirections[$code] = $redirection;
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFilters() :array {
    return $this->filtering ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function setFilters(array $filters) :self {
    $this->filtering = $filters;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRoles() :array {
    return $this->roles ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function setRoles(array $roles) :self {
    $this->roles = $roles;
    return $this;
  }

}
