<?php

namespace Drupal\site_integrator\Entity;

use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Provides an interface defining an integrated site.
 *
 * @ingroup site_integrator
 */
interface IntegratedSiteInterface extends ConfigEntityInterface {

  /**
   * Merge integration mode.
   *
   * Used when the HTML content of the integrated site is directly merged to the
   * Drupal page HTML content.
   */
  const INTEGRATION_MODE_MERGE = 1;

  /**
   * Internal iframe integration mode.
   *
   * Used when the HTML content of the integrated site is loaded in an iframe
   * using Javascript and a Javascript variable holding the HTML content.
   */
  const INTEGRATION_MODE_INTERNAL_IFRAME = 2;

  /**
   * External iframe integration mode.
   *
   * Used when the integrated site is loaded in an ifram using its base URL.
   */
  const INTEGRATION_MODE_EXTERNAL_IFRAME = 3;

  /**
   * Filter type for CSS.
   */
  const FILTER_TYPE_CSS = 'css';

  /**
   * Filter type for Javascript.
   */
  const FILTER_TYPE_JS = 'js';

  /**
   * Filter type for regular expressions.
   */
  const FILTER_TYPE_SELECTOR = 'selector';

  /**
   * Filter type for regular expressions.
   */
  const FILTER_TYPE_REGEXP = 'regexp';

  /**
   * Returns the integrated site base URL.
   *
   * @return string
   *   Base URL of the integrated site.
   */
  public function getBaseUrl() :string;

  /**
   * Sets the integrated site base URL.
   *
   * @param string $base_url
   *   The new base URL of the integrated site.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setBaseUrl(string $base_url) :self;

  /**
   * Returns the integrated site Drupal path.
   *
   * @return string
   *   Drupal path of the integrated site.
   */
  public function getPath() :string;

  /**
   * Sets the integrated site Drupal path.
   *
   * @param string $path
   *   The new path for the integrated site.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setPath(string $path) :self;

  /**
   * Returns integration mode.
   *
   * Returns one of the INTEGRATION_MODE_* constants.
   *
   * @return int
   *   The integration mode.
   */
  public function getIntegrationMode() :int;

  /**
   * Sets the integration mode.
   *
   * @param string $mode
   *   The new integration mode.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setIntegrationMode(int $mode) :self;

  /**
   * Returns the list of URLs that can passthrough.
   *
   * @return string[]
   *   The passthrough list.
   */
  public function getPassthroughList() :array;

  /**
   * Sets the passthrough.
   *
   * @param array $passthrough
   *   The new list of URLs that can passthrough.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setPassthroughList(array $passthrough) :self;

  /**
   * Returns the site integration status.
   *
   * @return bool
   *   True if the integration is enabled.
   */
  public function isEnabled() :bool;

  /**
   * Sets the integration status.
   *
   * @param bool $status
   *   Set to TRUE to enabled the integration or FALSE otherwise.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setEnabled(bool $status = TRUE) :self;

  /**
   * Tells if the site integration cannot be edited.
   *
   * @return bool
   *   True if the integration can be edited.
   */
  public function isEditable() :bool;

  /**
   * Sets the site integration edition status.
   *
   * @param bool $status
   *   Set to FALSE to disable user edition of the site integration config. The
   *   site integration becomes only editable by code. Set to TRUE, user can
   *   edit the configuration.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setEditable(bool $status = TRUE) :self;

  /**
   * Returns redirection for the given code or all the redirections.
   *
   * Redirections are keyed by the HTTP status code. There are two special
   * redirection keys: 'disabled' is used when the site integration is disabled,
   * and 'forbidden' when the site integration is not allowed to current user.
   * Generic HTTP codes can also be handled by "3xx", "4xx", and "5xx" keys.
   *
   * @param string|null $code
   *   The HTTP code that triggers the redirection. NULL to get all the
   *   redirections.
   *
   * @return array|string|null
   *   If $code is set, the associated redirection path/URL or NULL if not
   *   set. If $code is not set, returns an associative array with the
   *   redirections set.
   */
  public function getRedirection(?string $code = NULL) :array|string|null;

  /**
   * Sets the integration status.
   *
   * @param string $code
   *   The code triggering the redirection.
   * @param string|null $redirection
   *   The redirection path or URL or an empty value to disable the redirection.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setRedirection(string $code, ?string $redirection) :self;

  /**
   * Returns the list of content filters.
   *
   * @return array
   *   Each element of the array is an associative array with the following
   *   keys:
   *   - 'type': string of the type. One of 'css', 'js', or 'regexp' (see
   *     IntegratedSiteInterface::FILTER_TYPE_* constants).
   *   - 'filter': a filtering expression.
   *   - 'replacement': a replacement expression. Can be empty.
   */
  public function getFilters() :array;

  /**
   * Sets the content filters.
   *
   * @param array $filters
   *   The new content filters.
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setFilters(array $filters) :self;

  /**
   * Returns the roles allowed to use the site integration.
   *
   * @return string[]
   *   An array of role names.
   */
  public function getRoles() :array;

  /**
   * Sets the roles allowed to use the site integration.
   *
   * @param array $roles
   *   The list of roles allowed to use the site integration (when enabled).
   *
   * @return \Drupal\site_integrator\IntegratedSiteInterface
   *   Returns current instance.
   */
  public function setRoles(array $roles) :self;

}
