<?php

namespace Drupal\site_integrator\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\site_integrator\Entity\IntegratedSiteInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class IntegratedSiteFormBase.
 *
 * Base form for integrated sites.
 *
 * @ingroup site_integrator
 */
class IntegratedSiteFormBase extends EntityForm {

  /**
   * An entity query factory for the integrated site entity type.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * Construct the IntegratedSiteFormBase.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $entity_storage
   *   An entity query factory for the integrated site entity type.
   */
  public function __construct(EntityStorageInterface $entity_storage) {
    $this->entityStorage = $entity_storage;
  }

  /**
   * Factory method for IntegratedSiteFormBase.
   */
  public static function create(ContainerInterface $container) {
    $form = new static($container->get('entity_type.manager')->getStorage('integrated_site'));
    $form->setMessenger($container->get('messenger'));
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $insite = $this->entity;
    // Check if edition is allowed.
    if (!$insite->isEditable()) {
      return [
        '#type' => 'item',
        '#markup' => $this->t('This site integration is not editable.'),
      ];
    }

    // Build the form.
    $form = parent::buildForm($form, $form_state);

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable site integration'),
      '#default_value' => ($insite->isNew() || $insite->isEnabled()) ? 1 : NULL,
      '#description' => $this->t(
        'When disabled, the site page will either redirect to the "disabled" redirection page or return a 404 status (page not found).'
      ),
    ];
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $insite->label(),
      '#required' => TRUE,
    ];
    $form['id'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Machine name'),
      '#default_value' => $insite->id() ?? NULL,
      '#machine_name' => [
        'exists' => [$this, 'exists'],
        'replace_pattern' => '([^a-z0-9_]+)',
        'error' => 'The machine-readable name must be unique, and can only contain lowercase letters, numbers, and underscores.',
      ],
      '#disabled' => !$insite->isNew(),
    ];
    $form['base_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Base URL'),
      '#default_value' => $insite->getBaseUrl() ?? NULL,
      '#required' => TRUE,
    ];

    $form['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Drupal path'),
      '#default_value' => $insite->getPath() ?? NULL,
      '#required' => TRUE,
      '#description' => $this->t('Path that will display the integrated site. Must start with a slash and must not be already used by another part of the Drupal site.'),
    ];

    $form['mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Integration mode'),
      '#options' => [
        IntegratedSiteInterface::INTEGRATION_MODE_MERGE => $this->t('Merge site content'),
        IntegratedSiteInterface::INTEGRATION_MODE_INTERNAL_IFRAME => $this->t('Integrate into an IFrame as local content'),
        IntegratedSiteInterface::INTEGRATION_MODE_EXTERNAL_IFRAME => $this->t('Integrate into an IFrame as a regular external site'),
      ],
      '#default_value' => $insite->getIntegrationMode() ?? NULL,
      '#required' => TRUE,
    ];

    $form['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site frame height'),
      '#default_value' => $insite->get('height') ?? NULL,
      '#description' => $this->t('Enter a valid height for CSS (with unit). Leave empty to use default (automatic).'),
    ];

    $form['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site frame width'),
      '#default_value' => $insite->get('width') ?? NULL,
      '#description' => $this->t('Enter a valid width for CSS (with unit). Leave empty to use default (100%).'),
    ];

    // Passthrough.
    $passthrough = $insite->getPassthroughList();
    if (is_array($passthrough)) {
      $passthrough = implode("\n", $passthrough);
    }
    $form['passthrough_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('"Passthrough" paths'),
      '#default_value' => $passthrough,
      '#description' => $this->t('List of integrated site paths that will not be integrated but served directly (one by line). Note: only HTML content can be integrated, no need to list other types of content such as images.'),
    ];

    // Redirections.
    $redirections = $insite->getRedirection();
    if (!empty($redirections) && is_array($redirections)) {
      $redirections = implode(
        "\n",
        array_map(
          function ($code) use ($redirections) {
            return "$code|" . $redirections[$code];
          },
          array_keys($redirections)
        )
      );
    }
    $form['redirections'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Redirection paths and URLs'),
      '#default_value' => $redirections,
      '#description' => $this->t(
        'List of redirections. Each redirection consists of a code followed by a pipe sign and a redirection URL (ie. either a Drupal path or a full schema-qualified URL). Codes can be HTTP codes like "404" or "501" but also generic codes like "4xx". Specific codes like "404" will always have priority over generic codes like "4xx" if defined. There 2 additional special codes: "disabled" used when the integrated site is disabled and "forbidden" when Drupal forbids the access to the integrated site page to current Drupal user (not to be confused with a 403 status code returned by the integrated site itself). Redirections will redirect the Drupal page (and not the integrated site page).'
      ),
    ];

    // Filters.
    $filters = $form_state->get('filters') ?? $insite->getFilters();
    $form['filters'] = [
      '#type' => 'details',
      '#title' => $this->t('Filters'),
      '#open' => !empty($filters),
      '#tree' => TRUE,
      '#description' => $this->t(
        'Filters can be used to remove or replace parts of the integrated site content like CSS or Javascript files or other HTML elements.'
      ),
      '#attributes' => ['id' => 'site_integrator_filters'],
      '#states' => [
        'visible' => [
          ':input[id="edit-mode"]' => ['!value' => IntegratedSiteInterface::INTEGRATION_MODE_EXTERNAL_IFRAME],
        ],
      ],
    ];
    foreach ($filters as $i => $filter) {
      $form['filters'][$i] = $this->buildFilterForm($filter);
      $form['filters'][$i]['filter_remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove filter'),
        '#name' => 'filter_remove_' . $i,
        '#ajax' => [
          'callback' => [
            get_class($this),
            'buildAjaxFilterForm',
          ],
          'wrapper' => 'site_integrator_filters',
          'method' => 'replaceWith',
          'effect' => 'fade',
        ],
      ];

    }

    $form['filters']['filter_add_css'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add CSS filter'),
      '#name' => 'filter_add_css',
      '#ajax' => [
        'callback' => [
          get_class($this),
          'buildAjaxFilterForm',
        ],
        'wrapper' => 'site_integrator_filters',
        'method' => 'replaceWith',
        'effect' => 'fade',
      ],
    ];
    $form['filters']['filter_add_js'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Javascript filter'),
      '#name' => 'filter_add_js',
      '#ajax' => [
        'callback' => [
          get_class($this),
          'buildAjaxFilterForm',
        ],
        'wrapper' => 'site_integrator_filters',
        'method' => 'replaceWith',
        'effect' => 'fade',
      ],
    ];
    $form['filters']['filter_add_selector'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add HTML selector filter'),
      '#name' => 'filter_add_selector',
      '#ajax' => [
        'callback' => [
          get_class($this),
          'buildAjaxFilterForm',
        ],
        'wrapper' => 'site_integrator_filters',
        'method' => 'replaceWith',
        'effect' => 'fade',
      ],
    ];
    $form['filters']['filter_add_regexp'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add RegExp filter'),
      '#name' => 'filter_add_regexp',
      '#ajax' => [
        'callback' => [
          get_class($this),
          'buildAjaxFilterForm',
        ],
        'wrapper' => 'site_integrator_filters',
        'method' => 'replaceWith',
        'effect' => 'fade',
      ],
    ];

    // Role-based permissions.
    $roles = $insite->getRoles();
    sort($roles);
    $form['role_list'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Role-based access restriction'),
      '#default_value' => implode(', ', $roles),
      '#description' => $this->t(
        'Coma-separated list of roles allowed to use this site integration. Leave empty to disable restrictions.'
      ),
    ];

    return $form;
  }

  /**
   * Builds a filter form.
   *
   * @param array $filter
   *   A filter structure with 'type', 'filter' and 'replacement'.
   *
   * @return array
   *   A filter form element.
   */
  public function buildFilterForm(array $filter) :array {
    $filter_form = [
      '#type' => 'fieldset',
    ];
    $type = $filter['type'] ?? IntegratedSiteInterface::FILTER_TYPE_REGEXP;
    $filter_form['type'] = [
      '#type' => 'hidden',
      '#default_value' => $type,
    ];
    switch ($type) {
      case IntegratedSiteInterface::FILTER_TYPE_CSS:
        $filter_form['filter'] = [
          '#type' => 'textfield',
          '#title' => $this->t('CSS file to filter:'),
          '#attributes' => [
            'placeholder' => $this->t(
              'ex.: bootstrap.min.css'
            ),
          ],
          '#default_value' => $filter['filter'] ?? '',
        ];
        $filter_form['replacement'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Replacement CSS URL:'),
          '#description' => $this->t('Leave empty to just remove the CSS file.'),
          '#default_value' => $filter['replacement'] ?? '',
        ];
        break;

      case IntegratedSiteInterface::FILTER_TYPE_JS:
        $filter_form['filter'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Javascript file to filter:'),
          '#attributes' => [
            'placeholder' => $this->t(
              'ex.: jquery-1.2.3.min.js'
            ),
          ],
          '#default_value' => $filter['filter'] ?? '',
        ];
        $filter_form['replacement'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Replacement Javascript URL:'),
          '#description' => $this->t('Leave empty to just remove the Javascript file.'),
          '#default_value' => $filter['replacement'] ?? '',
        ];
        break;

      case IntegratedSiteInterface::FILTER_TYPE_SELECTOR:
        $filter_form['filter'] = [
          '#type' => 'textfield',
          '#title' => $this->t('HTML CSS selector:'),
          '#description' => $this->t('Enter a CSS selector to identify the HTML element(s) to remove or replace.'),
          '#attributes' => [
            'placeholder' => $this->t(
              'ex.: "#some_id", or ".some-class, nav", or "div[name="some_id"]"'
            ),
          ],
          '#default_value' => $filter['filter'] ?? '',
        ];
        $filter_form['replacement'] = [
          '#type' => 'textarea',
          '#title' => $this->t('Replacement HTML content:'),
          '#description' => $this->t('Leave empty to just remove the element.'),
          '#default_value' => $filter['replacement'] ?? '',
        ];
        break;

      case IntegratedSiteInterface::FILTER_TYPE_REGEXP:
      default:
        $filter_form['filter'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Regular expression:'),
          '#description' => $this->t('Enter a regular expression to match text to remove or replace. Do not include delimiters (delimiters used are tilds "~"). You can use capture parenthesis for replacement.'),
          '#field_prefix' => '~',
          '#field_suffix' => '~',
          '#attributes' => [
            'placeholder' => $this->t(
              'ex.: "My external app v(\d+\.\d+)"'
            ),
          ],
          '#default_value' => $filter['filter'] ?? '',
        ];
        $filter_form['replacement'] = [
          '#type' => 'textarea',
          '#title' => $this->t('Replacement HTML content:'),
          '#description' => $this->t('Leave empty to just remove the element.'),
          '#attributes' => [
            'placeholder' => $this->t(
              'ex.: "Working with app version \1"'
            ),
          ],
          '#default_value' => $filter['replacement'] ?? '',
        ];
        break;
    }
    return $filter_form;
  }

  /**
   * Handles AJAX filter form update.
   *
   * @param array $form
   *   The current form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object.
   *
   * @return array
   *   The filter part of the form to return as AJAX.
   */
  public static function buildAjaxFilterForm(
    array &$form,
    FormStateInterface &$form_state,
    Request $request
  ) {
    return $form['filters'];
  }

  /**
   * Checks for an existing integrated site.
   *
   * @param string|int $entity_id
   *   The entity ID.
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return bool
   *   TRUE if this format already exists, FALSE otherwise.
   */
  public function exists($entity_id, array $element, FormStateInterface $form_state) {
    $query = $this->entityStorage->getQuery();

    // Query the entity ID to see if its in use.
    $result = $query->condition('id', $element['#field_prefix'] . $entity_id)
      ->accessCheck()
      ->execute();

    // We don't need to return the ID, only if it exists or not.
    return (bool) $result;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    // Get the basic actins from the base class.
    $actions = parent::actions($form, $form_state);

    // Change the submit button text.
    $actions['submit']['#value'] = $this->t('Save');

    // Return the result.
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    // Check for filters update.
    $filters = [];
    foreach ($form_state->getValue('filters', []) as $filter) {
      // Skip submit buttons or other non-filter elements.
      if (!is_array($filter) || empty($filter['type'])) {
        continue;
      }
      $filters[] = [
        'type' => $filter['type'],
        'filter' => $filter['filter'],
        'replacement' => $filter['replacement'],
      ];
    }
    if (($trigger = $form_state->getTriggeringElement())
      && !empty($trigger['#name'])
      && (0 === strncmp($trigger['#name'], 'filter_', 7))
    ) {
      if (0 === strncmp($trigger['#name'], 'filter_remove_', 14)) {
        $index = substr($trigger['#name'], 14);
        unset($filters[$index]);
        $filters = array_values($filters);
      }
      elseif ('filter_add_css' === $trigger['#name']) {
        $filters[] = ['type' => IntegratedSiteInterface::FILTER_TYPE_CSS];
      }
      elseif ('filter_add_js' === $trigger['#name']) {
        $filters[] = ['type' => IntegratedSiteInterface::FILTER_TYPE_JS];
      }
      elseif ('filter_add_selector' === $trigger['#name']) {
        $filters[] = ['type' => IntegratedSiteInterface::FILTER_TYPE_SELECTOR];
      }
      elseif ('filter_add_regexp' === $trigger['#name']) {
        $filters[] = ['type' => IntegratedSiteInterface::FILTER_TYPE_REGEXP];
      }
      $form_state->setRebuild(TRUE);
    }
    $filters = array_filter(
      $filters,
      function ($filter) {
        return (
          !empty($filter['type'])
          && (!array_key_exists('filter', $filter) || !empty($filter['filter']))
        );
      }
    );
    // Update filters structure.
    $form_state->set('filters', $filters);
    $form_state->setValue('filtering', $filters);

    // Restructure "passthrough" parameter.
    $passthrough = $form_state->getValue('passthrough_list');
    if (!is_array($passthrough)) {
      $passthrough = array_filter(
        array_map('trim', explode("\n", $passthrough ?? '')),
        'strlen'
      );
    }
    $form_state->setValue('passthrough', $passthrough);

    // Check path...
    $path = $form_state->getValue('path');
    // Check for invalid characters.
    if (!preg_match('#^[.\w/\-]+$#', $path)) {
      $form_state->setErrorByName(
        'path',
        $this->t(
          'The given path (@path) contains unsupported characters. Allowed characters are a-z, 0-9, "_", "." and "-".',
          ['@path' => $path]
        )
      );
    }
    // Make sure the path begins with '/'.
    if ($path[0] !== '/') {
      $path = '/' . $path;
      $form_state->setValue('path', $path);
    }
    // Make sure the path is not already in use.
    if ($this->entity->isNew()
        && !empty(\Drupal::service('path.validator')->getUrlIfValid($path))
    ) {
      $form_state->setErrorByName(
        'path',
        $this->t(
          'The given path (@path) is already in use and will not be replaced.',
          ['@path' => $path]
        )
      );
    }

    // Adjust redirections structure.
    $redirections_str = $form_state->getValue('redirections', '');
    if (is_string($redirections_str)) {
      $redirections = [];
      foreach (array_filter(array_map('trim', explode("\n", $redirections_str))) as $redirection) {
        [$code, $url] = explode('|', $redirection, 2);
        if (!empty($url)) {
          $redirections[$code] = $url;
        }
      }
      $form_state->setValue('redirections', $redirections);
    }

    // Adjust roles structure.
    $roles = $form_state->getValue('role_list', '');
    $roles = array_filter(array_map('trim', explode(',', $roles)));
    $role_storage = \Drupal::service('entity_type.manager')
      ->getStorage('user_role');
    // Make sure given roles exist.
    $loaded_roles = $role_storage->loadMultiple($roles);
    $unexisting_roles = array_diff($roles, array_keys($loaded_roles));
    if (!empty($unexisting_roles)) {
      $form_state->setErrorByName(
        'role_list',
        $this->t(
          'The following roles have not been found: @roles.',
          ['@roles' => implode(', ', $unexisting_roles)]
        )
      );
    }
    $form_state->setValue('roles', $roles);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // EntityForm provides us with the entity we're working on.
    $insite = $this->getEntity();

    // Drupal already populated the form values in the entity object. Each
    // form field was saved as a public variable in the entity class. PHP
    // allows Drupal to do this even if the method is not defined ahead of
    // time.
    $status = $insite->save();

    // Grab the URL of the new entity. We'll use it in the message.
    $url = $insite->toUrl();

    // Create an edit link.
    $edit_link = Link::fromTextAndUrl($this->t('Edit'), $url)->toString();

    if ($status == SAVED_UPDATED) {
      // If we edited an existing entity...
      $this->messenger()->addMessage(
        $this->t(
          'Integrated site %label has been updated.',
          ['%label' => $insite->label()]
        )
      );
      $this->logger('site_integrator')->notice(
        'Integrated site %label has been updated.',
        ['%label' => $insite->label(), 'link' => $edit_link]
      );
    }
    else {
      // If we created a new entity...
      $this->messenger()->addMessage(
        $this->t(
          'Integrated site %label has been added.',
          ['%label' => $insite->label()]
        )
      );
      $this->logger('site_integrator')->notice(
        'Integrated site %label has been added.',
        ['%label' => $insite->label(), 'link' => $edit_link]
      );
    }

    // Rebuild route cache.
    \Drupal::service('router.builder')->rebuild();

    // Redirect the user back to the listing route after the save operation.
    $form_state->setRedirect('entity.integrated_site.list');

    return $status;
  }

}
