<?php

namespace Drupal\Tests\site_integrator\Functional;

use Drupal\Core\Url;
use Drupal\site_integrator\Entity\IntegratedSiteInterface;
use Drupal\Tests\BrowserTestBase;

/**
 * Ensure that the site_integrator forms work properly.
 *
 * @group site_integrator
 *
 * @ingroup site_integrator
 */
class IntegratedSiteTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Our module dependencies.
   *
   * @var string[]
   */
  protected static $modules = ['site_integrator'];

  /**
   * The installation profile to use with this test.
   *
   * @var string
   */
  protected $profile = 'minimal';

  /**
   * A user with administration permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $account;

  /**
   * Logs in as admin.
   */
  public function loginAdmin() {
    if (empty($this->account)) {
      // Create the user with all needed permissions.
      $this->account = $this->drupalCreateUser([
        // 'access administration pages',
        'administer integrated sites',
      ]);
    }
    $this->drupalLogin($this->account);
  }

  /**
   * Basic functional test for site integration.
   */
  public function testFunctional() {
    $assertion = $this->assertSession();
    $host = \Drupal::request()->getSchemeAndHttpHost();
    // Access list page.
    $path = Url::fromRoute('entity.integrated_site.list');
    error_log('entity.integrated_site.list');
    $this->drupalGet($path);
    // Restricted to admin.
    $assertion->statusCodeEquals(403);
    $this->loginAdmin();
    $this->drupalGet($path);
    $assertion->statusCodeEquals(200);

    // H1 title 'Integrated site'.
    $assertion->pageTextContains('Integrated site');
    // Empty list.
    $assertion->pageTextContains('There are no integrated site entities yet');
    // Tools menu.
    $assertion->linkExists('Integrated sites');
    // Add link.
    $assertion->linkExists('Add an integrated site');
    // Add form.
    $this->clickLink('Add an integrated site');
    $assertion->pageTextContains('Add an integrated site');
    $assertion->checkboxChecked('Enable site integration');
    $assertion->fieldExists('Label');
    $assertion->fieldExists('Base URL');
    $assertion->fieldExists('Drupal path');
    $assertion->optionExists('Integration mode', IntegratedSiteInterface::INTEGRATION_MODE_MERGE);
    $assertion->optionExists('Integration mode', IntegratedSiteInterface::INTEGRATION_MODE_INTERNAL_IFRAME);
    $assertion->optionExists('Integration mode', IntegratedSiteInterface::INTEGRATION_MODE_EXTERNAL_IFRAME);
    $assertion->fieldExists('Site frame height');
    $assertion->fieldExists('Site frame width');
    $assertion->fieldExists('"Passthrough" paths');
    $assertion->fieldExists('Redirection paths and URLs');
    $assertion->fieldExists('Role-based access restriction');
    $assertion->buttonExists('Add CSS filter');
    $assertion->buttonExists('Add Javascript filter');
    $assertion->buttonExists('Add HTML selector filter');
    $assertion->buttonExists('Add RegExp filter');
    $config = [
      'enabled' => TRUE,
      'label' => 'Test site',
      'id' => 'test_site',
      'base_url' => $host . '/',
      'path' => '/external',
      'mode' => IntegratedSiteInterface::INTEGRATION_MODE_INTERNAL_IFRAME,
      'height' => '',
      'width' => '',
      'passthrough_list' => '',
      'redirections' => '',
      'role_list' => '',
    ];
    $this->submitForm($config, 'Create integrated site');
    // Check update list.
    $this->drupalGet($path);
    // H1 title 'Integrated site'.
    $assertion->pageTextContains('Integrated sites');
    // Not empty list.
    $assertion->pageTextNotContains('There are no integrated site entities yet');
    $assertion->pageTextContains('Test site');
    $assertion->linkExists('/external');
    $assertion->linkExists($host . '/');
    // Edit.
    $this->clickLink('Edit');
    $assertion->pageTextContains('Edit integrated site');
    $assertion->checkboxChecked('Enable site integration');
    $assertion->fieldValueEquals('Label', 'Test site');
    $assertion->fieldValueEquals('Base URL', $host . '/');
    $assertion->fieldValueEquals('Drupal path', '/external');
    $assertion->fieldValueEquals('Integration mode', IntegratedSiteInterface::INTEGRATION_MODE_INTERNAL_IFRAME);
    $assertion->fieldValueEquals('Site frame height', '');
    $assertion->fieldValueEquals('Site frame width', '');
    $assertion->fieldValueEquals('"Passthrough" paths', '');
    $assertion->fieldValueEquals('Redirection paths and URLs', '');
    $assertion->fieldValueEquals('Role-based access restriction', '');
    $assertion->linkExists('Delete');
    $this->click('#edit-submit');
    $assertion->responseContains('<h1>Integrated sites</h1>');
    $assertion->pageTextNotContains('There are no integrated site entities yet');
    // Test integration.
    $this->drupalGet('/external/');
    $assertion->responseMatches('~<iframe\s+id="test_site_~');
    // Remove integration.
    $this->drupalGet($path);
    $this->clickLink('Edit');
    $this->clickLink('Delete');
    $assertion->pageTextContains('Are you sure');
    $assertion->linkExists('Cancel');
    $this->click('input[value="Delete integrated site"]');
    $assertion->pageTextContains('There are no integrated site entities yet');
  }

}
